program createtest

use dal

implicit none

real r1
integer, parameter :: nx = 250, ny = 250
integer, parameter :: nSource = 100
integer :: dims(2)
integer :: i,j,k
real, parameter  :: fracExtend=0.02, fracPoint = (1. - fracExtend)
real, parameter :: exposure = 1000.
real :: r, delta, xDash, yDash, ex1, ex2, F
real :: background
real :: flux(nSource)
real :: sigX(nSource)
real :: sigY(nSource)
real :: pa(nSource)
real :: xSource(nSource)
real :: ySource(nSource)
real :: rSourceFlux(nSource)
real, parameter :: pi = 3.141592654
type(DataSetT) :: outSet
type(ArrayT) :: outArray
real(kind=single), pointer :: image(:,:)
real :: minFlux, maxFlux
real, parameter :: nSigma = 10
real, parameter :: maxSig = 1., minSig=0.25
real :: x, y
real :: tot
real :: gasdev
real :: meanval


!
!-----------------------------------------------------------------
!
! AMK
!
    type(TableT) :: modesTableH                        ! Modes table dataset handle.
    type(ColumnT), dimension(5) :: modesColumnH        ! Modes column handle.
    integer(kind=int32), dimension(:), pointer :: modeP! Mode pointer.
    integer(kind=int32), dimension(:), pointer :: windowX0P           ! WindowX0 pointer.
    integer(kind=int32), dimension(:), pointer :: windowY0P           ! WindowY0 pointer.
    integer(kind=int32), dimension(:), pointer :: windowDXP           ! WindowDX pointer.
    integer(kind=int32), dimension(:), pointer :: windowDYP           ! WindowDY pointer.
    integer(kind=int32), dimension(1) :: columnDim                    ! Column dimension.

    type(ArrayT) :: qualArrayH                                        ! Quality Array handle.
    integer(kind=int32), dimension(2) :: qualArrayDim                 ! Quality array dimension.
!
!-----------------------------------------------------------------
!


dims(1) = nx
dims(2) = ny

call random_seed()

outSet = dataSet("testimage.fts",create)
outArray = addArray(outSet, "PRIMARY", real32, dims)
image => real32array2Data(outArray)

!
!------------------------------------------------------------------------
!
! AMK
!
! THis just to keep OMDETECT happy !!!!
!
! Add a MODES extension, detailing the science window parameters.
!
   modesTableH = addTable(outset, "MODES", 1)

   columnDim(1) = 1

   modesColumnH(1) = addColumn(modesTableH, "MODE", Integer32, "none",  columnDim)
   modeP => int32Data(modesColumnH(1))
   modeP = 0

   modesColumnH(2) = addColumn(modesTableH, "WINDOWX0", Integer32, "pixels",  columnDim)
   windowX0P => int32Data(modesColumnH(2))
   windowX0P = 0

   modesColumnH(3) = addColumn(modesTableH, "WINDOWY0", Integer32, "pixels",  columnDim)
   windowY0P => int32Data(modesColumnH(3))
   windowY0P = 0

   modesColumnH(4) = addColumn(modesTableH, "WINDOWDX", Integer32, "pixels",  columnDim)
   windowDXP => int32Data(modesColumnH(4))
   windowDXP = 0

   modesColumnH(5) = addColumn(modesTableH, "WINDOWDY", Integer32, "pixels",  columnDim)
   windowDYP => int32Data(modesColumnH(5))
   windowDYP = 0

   ! Release handles on data.
   call release(modesColumnH(1))
   call release(modesColumnH(2))
   call release(modesColumnH(3))
   call release(modesColumnH(4))
   call release(modesColumnH(5))
   call release(modesTableH)

! Add the quality extension to the image file.

   qualArrayDim(1) = 1
   qualArrayDim(2) = 1

   qualArrayH = addArray(outset, "QUALITY", Integer8, qualArrayDim)

! Release handles on data.
   call release(qualArrayH)

!
!------------------------------------------------------------------------
!

image = 0.

call random_number(background)

background=background * 1000.

minFlux = nSigma * sqrt(background)
maxFlux = minFlux * 100.

call random_number(xSource)
call random_number(ySource)
xSource = xSource * real(nx)
ySource = ySource * real(ny)

call random_number(flux)
flux = flux * (maxFlux-minFlux) + minFlux

call random_number(pa)
pa = pa * 2. * pi

do i = 1, nSource

  call random_number(r1)
  if (r1 > fracExtend) then

     sigX(i) = 0.5
     sigY(i) = 0.5

   else
     sigX(i) = 2.
     call random_number(sigY(i))
     sigY(i) = (0.5 + sigY(i)) * sigX(i)
  endif
enddo

rSourceFlux = 0
do i = 0, nx - 1
   do j = 0, ny - 1
      x = real(i)+0.5
      y = real(j)+0.5
      
      tot = 0.
      do k = 1, nSource
         r = sqrt((x-xSource(k))**2 + (y-ySource(k))**2)
         delta = atan2(y-ySource(k),x-xSource(k))

         xDash = r * cos(delta-pa(k))
         yDash = r * sin(delta-pa(k))

         ex1 = -(xDash**2/(2.*sigX(k)**2))

         ex2 = -(yDash**2/(2.*sigY(k)**2))


         f =  0.159154943 / (sigX(k) * sigY(k))
 
         f = f * flux(k) * exp(ex1) * exp(ex2)

         rSourceFlux(k) = rSourceFlux(k) + f
         tot = tot + f
      enddo
      image(i,j) = image(i,j) + tot
   enddo
enddo


do i = 0, nx-1
   do j = 0, ny-1
      f = gasdev()
      meanval = image(i,j) + background
      image(i,j) = meanval + f*sqrt(meanval) ! gaussian deviate
   enddo
enddo


call setAttribute(outSet, "EXPOSURE", exposure)
call setAttribute(outset, "BLANK", 999999)   ! Added by CB
call setAttribute(outSet, "BINAX1", 1)
call setAttribute(outSet, "BINAX2", 1)
call setAttribute(outSet, "BINBPE", .false.)
call setAttribute(outSet, "FILTER", "V")

call release(outArray)
call release(outSet)

open(20,file="testimage.lis",status="unknown",form="formatted")
write(20,*) nSource
do i = 1, nSource
  write(20,*) xSource(i), ySource(i), rSourceFlux(i)/exposure, &
  sigX(i), sigY(i), pa(i)
enddo
close(20)

end program createtest

real function gasdev() result(output)

  integer, save :: iset
  real :: fac, rsq, v1, v2, ranNumber
  real, save :: gset
  data iset/0/

  if(iset .eq. 0)then
     loop1: do
        call random_number(ranNumber)
        v1 = 2.*ranNumber-1.
        call random_number(ranNumber)
        v2 = 2.*ranNumber-1.
        rsq = v1**2 + v2**2
        if(rsq.ge.1.or.rsq.eq.0)then
           continue
        else
           exit loop1
        endif
     end do loop1
     fac = sqrt(-2.*log(rsq)/rsq)
     gset =v1*fac
     output = v2*fac
     iset=1
  else
     output = gset
     iset = 0
  endif
  return
end function gasdev





